"""Constants and allowed parameter values specified in the Userlist API."""

PUSH_URL = 'https://push.userlist.com/'



# Date formats
VALID_DATE_FORMATS = ['%Y-%m-%dT%H:%M:%S%z', '%Y-%m-%dT%H:%M:%S']


from requests.auth import AuthBase


class UserlistApiAuth(AuthBase):
    # Provided by Userlist: https://app.userlist.com/settings/push
    def __init__(self, push_key):
        self.push_key = push_key

    def __call__(self, request):
        request.headers.update(get_auth_headers(self.push_key))
        return request


def get_auth_headers(push_key):
    return {"Content-Type": "application/json; charset=utf-8", "Accept": "application/json", "Authorization": 'Push ' + push_key}



from __future__ import unicode_literals

import requests
import os
import sys
import time

sys.path.append(os.getcwd())


from userlist_python import const, utils
from userlist_python.userlist_auth import UserlistApiAuth
from userlist_python.userlist_exception import UserlistApiException


class UserlistApiClient(object):

    """The core client object used to fetch data from Userlist endpoints.

    :param push_key: Your API key, a length-32 UUID string provided for your Userlist account.
        You must configure your push key  `https://app.userlist.com/settings/push`.
    :type push_key: str

    :param session: An optional :class:`requests.Session` instance from which to execute requests.
        **Note**: If you provide a ``session`` instance, :class:`UserlistApiClient` will *not* close the session
        for you.  Remember to call ``session.close()``, or use the session as a context manager, to close
        the socket and free up resources.
    :type session: `requests.Session <https://2.python-requests.org/en/master/user/advanced/#session-objects>`_ or None
    """

    def __init__(self, push_key, session=None):
        self.auth = UserlistApiAuth(push_key=push_key)
        if session is None:
            self.request_method = requests
        else:
            self.request_method = session


    def push_users(
            self,
            identifier=None,
            email=None,
            signed_up_at=None,
            last_seen_at=None,
            properties=None,
            company=None,
            companies=None,
            relationships=None
    ):

        """Call the `/users` endpoint.

          Push users data into Userlist


          :param identifier: A unique identifier for that user within your application.
          :type identifier: str or None

          :param email: Email of a user
          :type email: str or None

          :param signed_up_at: Date of the last sign up. Timestamps are expected to be strings formatted according to ISO 8601: YYYY-MM-DDTHH:MM:SS+HH:MM. If no time zone is given, we’ll assume UTC.
          :type signed_up_at: str or None

          :param last_seen_at: Date of the last seen. Timestamps are expected to be strings formatted according to ISO 8601: YYYY-MM-DDTHH:MM:SS+HH:MM. If no time zone is given, we’ll assume UTC.
          :type last_seen_at: str or None

          :param properties: Properties of a user
          :type properties: dict or None

          :param company: Company details
          :type company: str or dict or None

          :param companies: Companies details
          :type companies: list or None

          :param relationships: Companies relationships
          :type relationships: list or dict or None

          :return: status code
          :rtype: int
          :raises UserlistApiException: If the ``"status"`` value different from 202 response code.
          """

        payload = {}


        # identifier
        if not identifier and not email:
            raise ValueError("Missing required parameter identifier or email")

        if identifier is not None:
            payload["identifier"] = identifier

        if email is not None:
            payload["email"] = email

        if signed_up_at is not None:
            payload["signed_up_at"] = signed_up_at

        if last_seen_at is not None:
            payload["last_seen_at"] = last_seen_at

        # properties
        if properties is not None:
            payload["properties"] = properties

        if company is not None:
            payload["company"] = company

        # Topic
        if companies is not None:
            payload['companies'] = companies

        if relationships is not None:
            payload['relationships'] = relationships

        # Send Request
        try:
            r = self.request_method.post(const.PUSH_URL + 'users', auth=self.auth, timeout=30, json=payload)
        except:
            raise UserlistApiException({'status': 408, 'code': 'requestTimeout',
                                        'errors': ['Your request exceeds 30 seconds, please verify your internet connection.']})


        # Check Status of Request
        if r.status_code < 200 or r.status_code >= 300:
            raise UserlistApiException(r.json())

        print(f'Successful push => {str(r.status_code)}')
        return r.status_code

    def delete_users(
            self,
            identifier
    ):


        """Call the `/users/{identifier}` endpoint.

          Delete a user from Userlist

          :param identifier: A unique identifier for that user within your application. (required)
          :type identifier: str

          :return: status code
          :rtype: int
          :raises UserlistApiException: If the ``"status"`` value different from 202 response code.
          """

        payload = {}


        # identifier
        if not identifier:
            raise ValueError("identifier is a required param. Please fill it in.")
        else:
            payload["identifier"] = identifier

        # Send Request
        try:
            r = self.request_method.delete(const.PUSH_URL + 'users/' + str(payload["identifier"]), auth=self.auth, timeout=30)
        except:
            raise UserlistApiException({'status': 408, 'code': 'requestTimeout',
                                        'errors': ['Your request exceeds 30 seconds, please verify your internet connection.']})
        # Check Status of Request
        if r.status_code < 200 or r.status_code >= 300:
            raise UserlistApiException(r.json())

        print(f'Successful delete => {str(r.status_code)}')
        return r.status_code


    def push_companies(
            self,
            identifier,
            name=None,
            signed_up_at=None,
            last_seen_at=None,
            properties=None,
            user=None,
            users=None,
            relationships=None

    ):

        """Call the `/companies` endpoint.

          Push users data into Userlist


          :param identifier: A unique identifier for that user within your application. (required)
          :type identifier: str

          :param name: Company Name
          :type name: str or None

          :param signed_up_at: Date of the last sign up. Timestamps are expected to be strings formatted according to ISO 8601: YYYY-MM-DDTHH:MM:SS+HH:MM. If no time zone is given, we’ll assume UTC.
          :type signed_up_at: str or None

          :param last_seen_at: Date of the last seen. Timestamps are expected to be strings formatted according to ISO 8601: YYYY-MM-DDTHH:MM:SS+HH:MM. If no time zone is given, we’ll assume UTC.
          :type last_seen_at: str or None

          :param properties: Properties of a user
          :type properties: dict or None

          :param user: User associated with company
          :type user: str or dict or None

          :param users: Users associated with company
          :type users: list or None

          :param relationships: Companies relationships
          :type relationships: list or dict or None

          :return: status code
          :rtype: int
          :raises UserlistApiException: If the ``"status"`` value different from 202 response code.
          """

        payload = {}


        # identifier
        if not identifier:
            raise ValueError("identifier is a required param. Please fill it in.")
        else:
            payload["identifier"] = identifier

        if name is not None:
            payload["name"] = name

        if signed_up_at is not None:
            payload["signed_up_at"] = signed_up_at

        if last_seen_at is not None:
            payload["last_seen_at"] = last_seen_at

        # properties
        if properties is not None:
            payload["properties"] = properties

        if user is not None:
            payload["user"] = user

        # Topic
        if users is not None:
            payload['users'] = users

        if relationships is not None:
            payload['relationships'] = relationships

        # Send Request
        try:
            r = self.request_method.post(const.PUSH_URL + 'companies', auth=self.auth, timeout=30, json=payload)
        except:
            raise UserlistApiException({'status': 408, 'code': 'requestTimeout',
                                        'errors': ['Your request exceeds 30 seconds, please verify your internet connection.']})
        # Check Status of Request
        if r.status_code < 200 or r.status_code >= 300:
            raise UserlistApiException(r.json())

        print(f'Successful push => {str(r.status_code)}')
        return r.status_code


    def delete_companies(
            self,
            identifier
    ):


        """Call the `/companies/{identifier}` endpoint.

          Delete a user from Userlist

          :param identifier: A unique identifier for that user within your application. (required)
          :type identifier: str

          :return: status code
          :rtype: int
          :raises UserlistApiException: If the ``"status"`` value different from 202 response code.
          """

        payload = {}


        # identifier
        if not identifier:
            raise ValueError("identifier is a required param. Please fill it in.")
        else:
            payload["identifier"] = identifier

        # Send Request
        try:
            r = self.request_method.delete(const.PUSH_URL + 'companies/' + str(payload["identifier"]), auth=self.auth, timeout=30)
        except:
            raise UserlistApiException({'status': 408, 'code': 'requestTimeout',
                                        'errors': ['Your request exceeds 30 seconds, please verify your internet connection.']})
        # Check Status of Request
        if r.status_code < 200 or r.status_code >= 300:
            raise UserlistApiException(r.json())

        print(f'Successful delete => {str(r.status_code)}')
        return r.status_code



    def push_relationships(
            self,
            user,
            company,
            properties=None

    ):

        """Call the `/relationships ` endpoint.

          Push relationship data into Userlist


          :param user: A unique identifier for user. (required)
          :type user: str

          :param company: A unique identifier for company. (required)
          :type company: str

          :param properties: Relationships
          :type properties: list or dict or None

          :return: status code
          :rtype: int
          :raises UserlistApiException: If the ``"status"`` value different from 202 response code.
          """

        payload = {}


        # identifier
        if not user:
            raise ValueError("user is a required param. Please fill it in.")
        else:
            payload["user"] = user

        if not company:
            raise ValueError("company is a required param. Please fill it in.")
        else:
            payload["company"] = company

        if properties is not None:
            payload['properties'] = properties

        # Send Request
        try:
            r = self.request_method.post(const.PUSH_URL + 'relationships', auth=self.auth, timeout=30, json=payload)
        except:
            raise UserlistApiException({'status': 408, 'code': 'requestTimeout',
                                        'errors': ['Your request exceeds 30 seconds, please verify your internet connection.']})
        # Check Status of Request
        if r.status_code < 200 or r.status_code >= 300:
            raise UserlistApiException(r.json())

        print(f'Successful push => {str(r.status_code)}')
        return r.status_code

    def delete_relationships(
            self,
            user,
            company
    ):


        """Call the `/relationships/{user-identifier}/{company-identifier}` endpoint.

          Delete a user from Userlist

          :param user: A unique identifier for that user within your application. (required)
          :type user: str

          :param company: A unique identifier for that company within your application. (required)
          :type company: str

          :return: status code
          :rtype: int
          :raises UserlistApiException: If the ``"status"`` value different from 202 response code.
          """

        payload = {}


        # identifier
        if not user:
            raise ValueError("user is a required param. Please fill it in.")
        else:
            payload["user"] = user


        if not company:
            raise ValueError("company is a required param. Please fill it in.")
        else:
            payload["company"] = company

        # Send Request
        try:
            r = self.request_method.delete(const.PUSH_URL + 'relationships/' + str(payload["user"]) + '/' + str(payload["company"]), auth=self.auth, timeout=30)
        except:
            raise UserlistApiException({'status': 408, 'code': 'requestTimeout',
                                        'errors': ['Your request exceeds 30 seconds, please verify your internet connection.']})
        # Check Status of Request
        if r.status_code < 200 or r.status_code >= 300:
            raise UserlistApiException(r.json())

        print(f'Successful delete => {str(r.status_code)}')
        return r.status_code


    def push_events(
            self,
            name,
            user=None,
            company=None,
            occured_at=None,
            properties=None

    ):
        """Call the `/events ` endpoint.

          Push events data into Userlist

          :param name: Name of your event. (required)
          :type name: str

          :param user: A unique identifier for user. (required)
          :type user: str

          :param company: A unique identifier for company. (required)
          :type company: str

          :param occured_at: Date of when the event occured. Timestamps are expected to be strings formatted according to ISO 8601: YYYY-MM-DDTHH:MM:SS+HH:MM. If no time zone is given, we’ll assume UTC.
          :type occured_at: str or None

          :param properties: properties
          :type properties: dict or None

          :return: status code
          :rtype: int
          :raises UserlistApiException: If the ``"status"`` value different from 202 response code.
          """

        payload = {}

        if not name:
            raise ValueError("name is a required param. Please fill it in.")
        else:
            payload["name"] = name

        if not user and not company:
            raise ValueError("either user or company parameter must be filled in. Please fill it in.")

        elif user and not company:
            payload["user"] = user

        elif not user and company:
            payload["company"] = company

        if occured_at is not None:
            payload["occured_at"] = occured_at

        if properties is not None:
            payload['properties'] = properties

        # Send Request
        try:
            r = self.request_method.post(const.PUSH_URL + 'events', auth=self.auth, timeout=30, json=payload)
        except:
            raise UserlistApiException({'status': 408, 'code': 'requestTimeout',
                                        'errors': ['Your request exceeds 30 seconds, please verify your internet connection.']})
        # Check Status of Request
        if r.status_code < 200 or r.status_code >= 300:
            raise UserlistApiException(r.json())

        print(f'Successful push => {str(r.status_code)}')
        return r.status_code



from __future__ import unicode_literals
from userlist_python import const
import datetime

import sys



def validate_string(given_string, variable_name):
    if is_valid_string(given_string):
        return given_string
    else:
        raise TypeError(f"{variable_name} parameter should be of type str")


def validate_email(email):
    if is_valid_string(email):
        if email.__contains__('@'):
            return email
        else:
            raise TypeError("email parameter must contain character '@'. ")
    else:
        raise TypeError("email parameter should be of type str")

def validate_date(given_date, variable_name):
    if is_valid_string(given_date):
        type_does_match = None
        for date_format in const.VALID_DATE_FORMATS:
            try:
                type_does_match = datetime.datetime.strptime(given_date, date_format)
            except:
                continue

        if not type_does_match:
            raise TypeError(f"{variable_name} parameter must be formatted according to ISO 8601"
                            f"- YYYY-MM-DDTHH:MM:SS+HH:MM. If no time zone is given, we’ll assume UTC.")
        else:
            return given_date
    else:
        raise TypeError(f"{variable_name} parameter should be of type str")


def validate_dict(given_dict, variable_name):
    if is_valid_dict(given_dict) and given_dict:
        return given_dict
    else:
        raise TypeError(f"{variable_name} parameter should be of type dict and not empty")


def validate_list_dict(given_list, variable_name):
    if is_valid_list(given_list) and given_list:
        for one_dict in given_list:
            if is_valid_dict(one_dict) and one_dict:
                continue
            else:
                raise TypeError(f"{variable_name} parameter should contain valid dict input")
        return given_list
    else:
        raise TypeError(f"{variable_name} parameter should be of type list and not empty")

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

if PY3:

    def is_valid_string(var):
        return isinstance(var, str)

    def is_valid_num(var):
        return isinstance(var, (int, float))

    def is_valid_list(var):
        return isinstance(var, list)

    def is_valid_boolean(var):
        return isinstance(var, bool)

    def is_valid_dict(var):
        return isinstance(var, dict)

elif PY2:

    def is_valid_string(var):
        return isinstance(var, basestring)

    def is_valid_num(var):
        return isinstance(var, (int, float, long))


else:

    def is_valid_string(var):
        raise SystemError("unsupported version of python detected (supported versions: 2, 3)")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['userlist_python']

package_data = \
{'': ['*']}

install_requires = \
['pendulum>=1.4,<2.0', 'requests>=2.24.0,<3.0.0']

setup_kwargs = {
    'name': 'userlist-python',
    'version': '0.2.0',
    'description': 'Official UserList SDK for Python',
    'long_description': '# Userlist SDK for Python\n\nThe official Python client library to manipulate [Userlist](https://userlist.com/) from your Python application.\n\nDocumentation is identical with the API documentation. The same parameters and filters are available.\nAnd the same response structure. You can have a look at [Docs](https://userlist.com/docs/getting-started/integration-guide/#setting-up-the-integration).\n\n## Authentication\n\nThe Authentication is done via the `push_key` variable.\n\nCheck your Push key at [Userlist Settings](https://app.userlist.com/settings/push).\n\n## Installation\n```pip install userlist```\n\n## Quick Start\nImport installed package.\n\n`````from userlist_python import UserlistApiClient`````\n\nInit the instance with an API key given after registration.\n\n````userlist_client = UserlistApiClient(\'YOUR_API_KEY\')````\n\n## Endpoints\nAn instance of `UserlistApiClient` has all main methods that correspond to endpoints available for UserList API.\n\n### Tracking user data (/users)\nUser data can be tracked by sending POST requests to `https://push.userlist.com/users`.\nThe only required parameter is identifier which is a unique identifier for that user within your application.\nThis can either be the user’s primary key in your database, a generated tracking identifier or their email address\n(we don’t recommend using email address though, because it’s less reliable). Whatever you choose, make please keep\nin mind that it’ll be the way Userlist identifies this user moving forward.\n\n```\nresponse = userlist_client.push_users(\n    identifier="user_test",\n    email=\'test@example.net\',\n    properties={"first_name": "Test2","last_name": "Testing2"}\n)\n```\n\n### Deleting users (/users/{{identifier}})\nYou can remove user data by sending a DELETE request to `https://push.userlist.com/users/{{identifier}}`.\nThe identifier is the same one you sent when sending the user data initially. We’ll process your deletion request within\na few of moments and remove the user, all their events, as well as all their messages. All campaigns will immediately\nbe stopped. If you send any data or event with this property after requesting a deletion, we’ll treat it as fresh and\ncreate a new user.\n\n```\nresponse = userlist_client.delete_users(\n    \'user_test\'\n)\n\n ```\n\n### Tracking company data (/companies)\nCompany data can be tracked by sending POST requests to `https://push.userlist.com/companies`.\nThe only required parameter is identifier which is a unique identifier for that company within your application.\nThis can either be the company\'s primary key in your database, or some kind of generated tracking identifier.\nWhatever you choose, make please keep in mind that it’ll be the way Userlist identifies this company moving forward.\n```\nresponse = userlist_client.push_companies(\n    identifier="company_test",\n    name=\'Example, Inc.\',\n    properties={\n    "industry": "Testing",\n    "billing_plan": "enterprise"\n  },\n    relationships=[\n    {\n      "user": "user_test",\n      "properties": {\n        "role": "owner"\n      }\n    }\n  ]\n)\n ```\n\n### Deleting Companies (/companies/{{identifier}})\nYou can remove company data by sending a DELETE request to `https://push.userlist.com/companies/{{identifier}}`.\nThe identifier is the same one you sent when sending the company data initially. We’ll process your deletion request\nwithin a few of moments and remove the company, all its events, as well as all relationships to users. The users that\nwhere part of that company are not deleted automatically. If you send any data or event with this company after\nrequesting a deletion, we’ll treat it as fresh and create a new company record.\n\n```\nresponse = userlist_client.delete_companies(\n    \'company_test\'\n)\n ```\n\n### Tracking relationships (/relationships)\nUserlist allows you to track relationships between users and companies. It\'s possible to track many-to-many\nrelationships between users and companies. Creating a relationship needs at least two pieces of information:\na user and a company.\n```\nresponse = userlist_client.push_relationships(\n    user="user_test",\n    company=\'company_test\',\n    properties={\n    "role": "admin"\n  }\n)\n ```\n\n### Deleting relationships (/relationships/{{user-identifier}}/{{company-identifier}})\nYou can remove a relationship data by sending a DELETE request to\n`https://push.userlist.com/relationships/{{user-identifier}}/{{company-identifier}}`. We’ll process your deletion\nrequest within a few of moments and remove the relationship between this user and this company. Both the associated user\nand company are not deleted automatically.\n```\nresponse = userlist_client.delete_relationships(\n    \'user_test\',\n    \'company_test\'\n)\n ```\n\n### Tracking events (/events)\nSimilar to tracking user or company data, tracking events works by sending POST requests to `https://push.userlist.com/events`.\nTracking an event requires at least two pieces of information: a name, and a user or company identifier.\nOther parameters are optional.\n```\nresponse = userlist_client.push_events(\n    name="product_purchased",\n    company=\'company_test\',\n    properties={\n    "product": "Flowers",\n    "price": "$12.99"\n  }\n)\n ```\n\n## Feedback\n\nFeel free to contact us if you have spot a bug or have any suggestion at benedikt`[at]`benediktdeicke.com\n',
    'author': 'Maksym Sugonyaka',
    'author_email': 'maksym@newscatcherapi.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://userlist.com/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



class UserlistApiException(Exception):
    """Represents an ``status`` response status value from Userlist API."""

    def __init__(self, exception):
        self.exception = exception

    def get_exception(self):
        return self.exception

    def get_status(self):
        if self.exception["status"]:
            return self.exception["status"]

    def get_code(self):
        if self.exception["code"]:
            return self.exception["code"]

    def get_errors(self):
        if self.exception["errors"]:
            return self.exception["errors"]
